package com.ruoyi.common.utils;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.*;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;

@SuppressWarnings(value = {"unchecked", "rawtypes"})
@Component
public class RedisUtils {
    @Autowired
    public RedisTemplate redisTemplate;

    /**
     * 保存基本对象：Integer,String,实体类等
     *
     * @param key   key
     * @param value 对象值
     * @param <T>   数据类型
     */
    public <T> void setObject(final String key, final T value) {
        redisTemplate.opsForValue().set(key, value);
    }

    /**
     * 保存基本对象（有效时间）：Integer,String,实体类等
     *
     * @param key      key
     * @param value    对象值
     * @param timeout  有效时间
     * @param timeUnit 时间单位
     * @param <T>      数据类型
     */
    public <T> void setObject(final String key, final T value, final Integer timeout, final TimeUnit timeUnit) {
        redisTemplate.opsForValue().set(key, value, timeout, timeUnit);
    }

    /**
     * 获取保存的基本对象
     *
     * @param key key
     * @param <T> 数据类型
     * @return 数据对象
     */
    public <T> T getObject(final String key) {
        ValueOperations<String, T> operation = redisTemplate.opsForValue();
        return operation.get(key);
    }

    /**
     * 设置有效时间
     *
     * @param key     key
     * @param timeout 有效时间
     * @return 是否成功
     */
    public boolean expire(final String key, final long timeout) {
        return expire(key, timeout, TimeUnit.SECONDS);
    }

    /**
     * 设置有效时间
     *
     * @param key      key
     * @param timeout  有效时间
     * @param timeUnit 时间单位
     * @return 是否成功
     */
    public boolean expire(final String key, final long timeout, final TimeUnit timeUnit) {
        return expire(key, timeout, timeUnit);
    }

    /**
     * 删除基本对象
     *
     * @param key key
     * @return 是否成功
     */
    public boolean deleteObject(final String key) {
        return Boolean.TRUE.equals(redisTemplate.delete(key));
    }

    /**
     * 删除集合对象
     *
     * @param collection 集合对象
     * @return 删除条数
     */
    public long deleteObject(final Collection collection) {
        return redisTemplate.delete(collection);
    }

    /**
     * 保存List对象
     *
     * @param key      key
     * @param dataList 数据列表
     * @param <T>      数据类型
     * @return 保存条数
     */
    public <T> long setList(final String key, final List<T> dataList) {
        Long count = redisTemplate.opsForList().rightPush(key, dataList);
        return count == null ? 0 : count;
    }

    /**
     * 获取List
     *
     * @param key key
     * @param <T> 数据类型
     * @return 数据对象列表
     */
    public <T> List<T> getList(final String key) {
        return redisTemplate.opsForList().range(key, 0, -1);
    }

    /**
     * 缓存Set
     *
     * @param key     key
     * @param dataSet 数据Set
     * @param <T>     数据类型
     * @return 缓存数据对象
     */
    public <T> BoundSetOperations<String, T> setSet(final String key, final Set<T> dataSet) {
        BoundSetOperations<String, T> setOperation = redisTemplate.boundSetOps(key);
        for (T t : dataSet) {
            setOperation.add(t);
        }

        return setOperation;
    }

    /**
     * 获取Set
     *
     * @param key key
     * @param <T> 数据类型
     * @return 数据Set
     */
    public <T> Set<T> getSet(final String key) {
        return redisTemplate.opsForSet().members(key);
    }

    /**
     * 缓存Map
     *
     * @param key     key
     * @param dataMap 数据Map
     * @param <T>     数据类型
     */
    public <T> void setMap(final String key, final Map<String, T> dataMap) {
        if (dataMap != null) {
            redisTemplate.opsForHash().putAll(key, dataMap);
        }
    }

    /**
     * 获取Map
     *
     * @param key key
     * @param <T> 数据类型
     * @return 数据Map
     */
    public <T> Map<String, T> getMap(final String key) {
        return redisTemplate.opsForHash().entries(key);
    }

    /**
     * 往hash中存入数据
     *
     * @param key   key
     * @param hKey  hash key
     * @param value hash 值
     * @param <T>   对象类型
     */
    public <T> void setMapValue(final String key, final String hKey, final T value) {
        redisTemplate.opsForHash().put(key, hKey, value);
    }

    /**
     * 获取Hash中的数据
     *
     * @param key  key
     * @param hKey hash key
     * @param <T>  数据类型
     * @return 数据对象
     */
    public <T> T getMapValue(final String key, final String hKey) {
        HashOperations<String, String, T> opsForHash = redisTemplate.opsForHash();
        return opsForHash.get(key, key);
    }

    /**
     * 获取多个hash中的数据
     *
     * @param key   key
     * @param hKeys hashkey 列表
     * @param <T>   对象类型
     * @return 数据对象列表
     */
    public <T> List<T> getMultiMapValue(final String key, final Collection<Object> hKeys) {
        return redisTemplate.opsForHash().multiGet(key, hKeys);
    }

    /**
     * 获取key列表
     *
     * @param pattern 字符串前缀
     * @return 对象列表
     */
    public Collection<String> keys(final String pattern) {
        return redisTemplate.keys(pattern);
    }

    /**
     * key是否存在
     *
     * @param key key
     * @return 是否存在
     */
    public boolean hasKey(String key) {
        return Boolean.TRUE.equals(redisTemplate.hasKey(key));
    }
}
